/********************************************************************
 * 2D Map for Radarcape
 ********************************************************************
 * Guenter Koellner Embedded Development GmbH
 *
 * Am Rain 24
 * 85256 Vierkirchen
 * Geschaeftsfuehrer Guenter Koellner
 *
 * http://www.modesbeast.com
 ********************************************************************/

var Map=null;
var Planes={};
var Filters={altitude: [undefined, undefined], speed: [undefined, undefined], distance: [undefined, undefined], hex: [undefined, undefined]};
var NumPlanes = 0;
var Selected=null;
var RangeCircle={};
var CircleStep=60;
var NauticalMiles_to_Meters=1852;
var infoboxmode=0; /* 0=Aircraft Info; 1=Config */

var flightPath;


function showInfoBox()
{
    document.getElementById('gmap_info').style.visibility='visible';
}


function hideInfoBox()
{
    document.getElementById('gmap_info').style.visibility='hidden';
}


function doRightClick()
{
    infoboxmode=1;
    refreshGeneralInfo();
    refreshSelectedInfo();
    showInfoBox();
}


function get_image_object(ac_category, ac_type)
{
   var retVal = acImagePathsR["$A320"];

   /* Assign a/c image based on a/c type */

   if(ac_category.length > 0)
   {
      if(ac_category == "A1") /* Light */
      {
        retVal = acImagePathsR["$2"];
      }
      else if(ac_category == "A2") /* Medium */
      {
        retVal = acImagePathsR["$3"];
      }
      else if(ac_category == "A3") /* Heavy */
      {
         retVal = acImagePathsR["$4"];
      }
      else if(ac_category == "A4") /* High-Vortex */
      {
         retVal = acImagePathsR["$4"];
      }
      else if(ac_category == "A5") /* Very-Heavy */
      {
         retVal = acImagePathsR["$A380"];
      }
      else if(ac_category == "A6") /* High-Performance/High-Speed */
      {
         retVal = acImagePathsR["$TORN"];
      }
      else if(ac_category == "A7") /* Rotorcraft */
      {
         retVal = acImagePathsR["$7"];
      }
   }

   /* Assign a/c image based on ICAO type code */
   if(ac_type.length > 0)
   {
      for (var i = 0; i < ac_symbol_map.length; i++)
      {
         var ac_symbol = ac_symbol_map[i];
         if (ac_type.indexOf(ac_symbol.id) == 0) {
            retVal = acImagePathsR[ac_symbol.sym];
            break;
         }
      }
   }

   return retVal;
}


function getIconForPlane(plane)
{

      var rFill = 255; gFill = 255; bFill = 0;
      var rBorder = 0, gBorder = 0, bBorder = 255;

      /* Colors for ADS-B/MLAT */
      if (plane.src == "M")
      {
         rBorder = 200; gBorder = 0; bBorder = 0;
      }
      else
      {
         rBorder = 64; gBorder = 64; bBorder = 64;    
      }

      /* Change color for climb/decent */
      if (plane.vrt > 300)
      {
          /*rFill = 25; gFill = 165; bFill = 166;*/
          rFill = 53; gFill = 235; bFill = 244;
      }
      else if (plane.vrt < -300)
      {
          /* original A320: rFill = 148; gFill = 91; bFill = 1; */
          /*rFill = 228; gFill = 137; bFill = 0;*/
          rFill = 185; gFill = 92; bFill = 0;
      }

      if (plane.timeout == 1) /* after timeout 1 */
      {
          rFill = 128; gFill = 128; bFill = 128;  /* dark gray */
          rBorder = rFill; gBorder=gFill; bBorder=bFill;
      }

      var maxalt = 40000; /* Max altitude in the average case */
      var invalt = maxalt-plane.alt;
      var selected = (Selected == plane.hex);
      var imageObj = get_image_object(plane.cat, plane.typ);

      var object_box = Raphael.pathBBox(imageObj.path);
      var offset_x = (object_box.x + object_box.x2) / 2;
      var offset_y = (object_box.y + object_box.y2) / 2;

      if (invalt < 0) invalt = 0;

      var strokew = 1;
      if (selected) { strokew = 2; }
      if (plane.timeout == 2) { strokew = 0.333; }   /* light on  timeout 2 */

      var ac_symbol = {
         strokeColor: 'rgb('+rBorder+','+gBorder+','+bBorder+')',
         strokeWeight: strokew,
         path: imageObj.path,
         scale: imageObj.scale,
         fillColor: 'rgb('+rFill+','+gFill+','+bFill+')',
         fillOpacity: 0.9,
         anchor: new google.maps.Point(offset_x, offset_y),
         rotation: (plane.trk + imageObj.rotcorr) % 360
      };

      var myZoom = Map.getZoom();
      if((myZoom <= 14) && ((6 < myZoom))) {
         ac_symbol.scale = ac_symbol.scale + 0.2 * ((myZoom-10)/4);
      }
      else if(myZoom < 14) {
         ac_symbol.scale = ac_symbol.scale - 0.2
      }
      else {
         ac_symbol.scale = ac_symbol.scale + 0.2
      }

      return ac_symbol;
}


function selectPlane()
{
      if (!Planes[this.planehex])return;
      var Old_Selected = Selected;
      Selected = this.planehex;
      if (Planes[Old_Selected])
      {
         /* Remove the highlight in the previously selected plane. */
         Planes[Old_Selected].marker1.setIcon(getIconForPlane(Planes[Old_Selected]));
         labelSetStyle(Planes[Old_Selected], Planes[Old_Selected].marker1);
      }
      Planes[Selected].marker1.setIcon(getIconForPlane(Planes[Selected]));
      labelSetStyle(Planes[Selected],Planes[Selected].marker1);
      infoboxmode=0;
      refreshSelectedInfo();
      fetchTrackData();
      showInfoBox();
}


function deselectPlane()
{
     if (Selected) {
         var Old_Selected = Selected;
         Selected = null;
         Planes[Old_Selected].marker1.setIcon(getIconForPlane(Planes[Old_Selected]));
         labelSetStyle(Planes[Old_Selected],Planes[Old_Selected].marker1);
         var i = document.getElementById('selinfo');
         i.innerHTML = "<p>Click on an aircraft for more info.</p>";
         flightPath.setPath([]);
         flightPath.setMap(null);
         hideInfoBox();
      }
}


function drawRangeRings()
{
    // Draw Circles
    var circleData = {
      strokeColor: '#FFFFFF',
      strokeOpacity: 0.6,
      strokeWeight: 1,
      fillColor: '#00FFFF',
      fillOpacity: 0.0,
      map: Map,
      center: new google.maps.LatLng(CenterLat, CenterLon),
      radius: 1 * CircleStep * NauticalMiles_to_Meters
    };

    RangeCircle[0] = new google.maps.Circle(circleData);

    circleData.radius = 2 * CircleStep * NauticalMiles_to_Meters;
    RangeCircle[1] = new google.maps.Circle(circleData);

    circleData.radius = 3 * CircleStep * NauticalMiles_to_Meters;
    RangeCircle[2] = new google.maps.Circle(circleData);

    circleData.radius = 4 * CircleStep * NauticalMiles_to_Meters;
    RangeCircle[3] = new google.maps.Circle(circleData);

    circleData.radius = 3.7 * CircleStep * NauticalMiles_to_Meters;
    circleData.strokeOpacity = 0.0;
    RangeCircle[4] = new google.maps.Circle(circleData);
    RangeCircle[4].setVisible(false);


    // Draw Tick Lines
    /*var lineObj = new Array(12);

    var innerNorth = RangeCircle[4].getBounds().getNorthEast().lat();
    var outerNorth = RangeCircle[3].getBounds().getNorthEast().lat();
    var innerEast  = RangeCircle[4].getBounds().getNorthEast().lng();
    var outerEast  = RangeCircle[3].getBounds().getNorthEast().lng();

    for(i = 0; i < lineObj.length; i++)
    {
      var radiant = i * 30 * (Math.PI / 180);

      var innerLat = CenterLat + ((innerNorth - CenterLat) * Math.cos(radiant));
      var innerLon = CenterLon + ((innerEast  - CenterLon) * Math.sin(radiant));
      var outerLat = CenterLat + ((outerNorth - CenterLat) * Math.cos(radiant));
      var outerLon = CenterLon + ((outerEast  - CenterLon) * Math.sin(radiant));

      var lineCoordinates = [
        new google.maps.LatLng(outerLat, outerLon),
        new google.maps.LatLng(innerLat, innerLon)
      ];

      lineObj[i] = new google.maps.Polyline({
        path: lineCoordinates,
        geodesic: true,
        strokeColor: '#FFFFFF',
        strokeOpacity: 0.7,
        strokeWeight: 2
      });

      lineObj[i].setMap(Map);
    }*/
}


function sortPlaneSymbolList(a, b)
{
      var al = a.id;
      var bl = b.id;

      if (al < bl) return 1;
      if (al > bl) return -1;
      return 0;
}


/* Info box in div gmap_info - 0=Aircraft Info 1=Config */
function refreshGeneralInfo()
{
      var i = document.getElementById('geninfo');
      i.innerHTML = NumPlanes+' aircraft listed';
      if (Selected == null)
      {
        if (flightPath.getPath() != null)
        {
          flightPath.setMap(null);
        }
      }
}


function refreshSelectedInfo()
{
      var i = document.getElementById('selinfo');
     if (infoboxmode==1) {
       i.innerHTML = "<p>Select Aircraft on Map</p>";
       return;
     }

      var p = Planes[Selected];

      if (p)
      {
        var html = '';

        html += '<table class=\"infotable\">';

        /* Row 2 */
        html += '  <tr class="infolabel">';
        html += '    <td colspan="2">Callsign</td>';
        html += '  </tr>';
        html += '  <tr class="infovalue">';
        if (p.fli.length) {
          html += '    <td colspan="2">'+ p.fli +'&nbsp;</td>';
        } else {
          html += '    <td colspan="2">&nbsp;</td>';
        }
        html += '  </tr>';

        /* Row 1 */
        html += '  <tr class="infolabel">';
        html += '    <td width="50%">ICAO</td>';
        html += '    <td width="50%">Registration</td>';
        html += '  </tr>';
        html += '  <tr class="infovalue">';
        html += '    <td>'+ p.hex +'&nbsp;</td>';
        if(ActivateLink) {
          html += '    <td><a href="http://www.planespotters.net/Aviation_Photos/search.php?tag='
          + encodeURIComponent(p.reg) + '" target="_blank">' + p.reg + '</a>&nbsp;</td>';
        } else {
          html += '    <td>' + p.reg + '&nbsp;</td>';
        }
        html += '  </tr>';

        /* Row 4 */
        html += '  <tr class="infolabel">';
        html += '    <td>Type</td>';
        html += '    <td>Category</td>';
        html += '  </tr>';
        html += '  <tr class="infovalue">';
        html += '    <td>' + p.typ + '&nbsp;</td>';
        html += '    <td>' + p.cat + '&nbsp;</td>';
        html += '  </tr>';

        /* Row 5 */
        html += '  <tr class="infolabel">';
        html += '    <td>Altitude</td>';
        html += '    <td>Speed</td>';
        html += '  </tr>';
        html += '  <tr class="infovalue">';
        html += '    <td>' + p.alt + ' ft</td>';
        html += '    <td>' + p.spd + ' kt</td>';
        html += '  </tr>';

        /* Row 6 */
        html += '  <tr class="infolabel">';
        html += '    <td>Vertical Rate</td>';
        html += '    <td>Track</td>';
        html += '  </tr>';
        html += '  <tr class="infovalue">';
        if(p.vrt) {
          html += '    <td>' + p.vrt + ' ft/min</td>';
        } else {
          html += '    <td></td>';
        }
        html += '    <td>' + p.trk + '&deg;</td>';
        html += '  </tr>';

        /* Row 7 */
        html += '  <tr class="infolabel">';
        html += '    <td>Latitude</td>';
        html += '    <td>Longitude</td>';
        html += '  </tr>';
        html += '  <tr class="infovalue">';
        html += '    <td>' + p.lat + '&deg;</td>';
        html += '    <td>' + p.lon + '&deg;</td>';
        html += '  </tr>';

        /* Row 3 */
        html += '  <tr class="infolabel">';
        html += '    <td>Origin</td>';
        html += '    <td>Destination</td>';
        html += '  </tr>';
        html += '  <tr class="infovalue">';
        html += '    <td>' + p.org + '&nbsp;</td>';
        html += '    <td>' + p.dst + '&nbsp;</td>';
        html += '  </tr>';

        /* Row 8 */
        html += '  <tr class="infolabel">';
        html += '    <td>Distance</td>';
        html += '    <td>Source</td>';
        html += '  </tr>';
        html += '  <tr class="infovalue">';
        html += '    <td>' + p.dis + ' ' + DistanceUnit + '</td>';
        html += '    <td>' + p.src + '</td>';
        html += '  </tr>';

        /* Row 9 */
        html += '  <tr class="infolabel">';
        html += '    <td>Last Signal</td>';
        html += '    <td>&nbsp;</td>';
        html += '  </tr>';
        html += '  <tr class="infovalue">';
        html += '    <td>' + p.lla + ' sec</td>';
        html += '    <td>&nbsp;</td>';
        html += '  </tr>';

        html += '</table>';

        i.innerHTML = html;
      }
      else
      {
        /* Remove Selected Info */
        i.innerHTML = "<p>Click on an aircraft for more info.</p>";
      }
}


/*************** Label formatting **************/

function labelDataFlight(plane)
{
      if (plane.fli == "") return "~" + plane.hex;
      return plane.fli;
}


function labelDataVerticalRate(plane)
{
        var s = "";

        // Vertical Rate
        if (plane.vrt > -9000) {
          if (plane.vrt < -300) s =       Math.round(plane.vrt/100);
          if (plane.vrt >  300) s = "+" + Math.round(plane.vrt/100);
        }

        return s;
}


function labelDataAltitude(plane)
{
      var s = "";

      // Aircraft on Ground
      if (plane.gda == "G")
      {
        s = "GND";
      }
      else
      {
        // Altitude
        s = Math.round(plane.alt/100);
        if (s < 0)
          s = "-0" + -1*s;
        else {
          if (s < 10)  s = "00" + s; else
          if (s < 100) s = "0" + s; 
        }
      }
      return s;
}


function labelDataSpeed(plane)
{
      return Math.round(plane.spd/10);
}


function labelDataTable(plane)
{
 //   var labelDataTab = "<table class=\"aclabel\"><tr><td class=\"flight\" colspan=\"3\">" + 
 //                      labelDataFlight(plane) + 
 //                      "</td></tr><tr><td class=\"altitude\">" +
 //                      labelDataAltitude(plane) + 
 //                      "</td><td class=\"verticalRate\">" +
 //                      labelDataVerticalRate(plane) + 
 //                      "</td><td class=\"speed\">" +
 //                      labelDataSpeed(plane) + 
 //                      "</td></tr></table>";

	var labelDataTab=labelDataFlight(plane)+"<br/>"+labelDataAltitude(plane)+" "+
	  labelDataVerticalRate(plane)+" "+labelDataSpeed(plane);
    return labelDataTab;
}


function labelSetAnchor(plane, marker, offset)
{
      if (plane.trk > 315) { marker.set("labelAnchor", new google.maps.Point( 66, -1 + offset)); return; }
      if (plane.trk < 45)  { marker.set("labelAnchor", new google.maps.Point( 66, -1 + offset)); return; }
      if (plane.trk < 135) { marker.set("labelAnchor", new google.maps.Point( 37, -1 + 28 + offset)); return; }
      if (plane.trk < 225) { marker.set("labelAnchor", new google.maps.Point(-17, -1 + offset)); return; }
      marker.set("labelAnchor", new google.maps.Point(37, -1 - 28 + offset));
}


function labelSetStyle(plane, marker)
{
      if (plane.hex == Selected) {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'greenyellow'}); return; }
      if (plane.squ == "7700")   {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'orangered'}); return; }
      if (plane.squ == "7600")   {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'orangered'}); return; }
      if (plane.squ == "7500")   {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'orangered'}); return; }
      if (plane.timeout == 1)    {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'silver'}); return; }
      if (plane.timeout == 2)    {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'tan'}); return; }
      if (plane.src == "M")      {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'orange'}); return; }
      marker.set("labelStyle", {opacity: 0.6, 'background-color': 'gold'});
}


function fetchTrackData()
{
      if(Selected != null)
      {
        var FlightPathURL = '/flightpath.json?icao=' + Selected;

        $.getJSON(FlightPathURL, function(data)
        {
          var trackpoints = new Array();

          for (var j=0; j < data.length; j++)
          {
            trackpoints.push( new google.maps.LatLng(data[j].lat, data[j].lon) );
          }

          if(trackpoints.length > 0)
          {
            flightPath.setPath(trackpoints);
            flightPath.setMap(Map);
          }
        });
      }
}


function fetchData()
{
    var swlat = Map.getBounds().getSouthWest().lat().toFixed(2);
    var swlon = Map.getBounds().getSouthWest().lng().toFixed(2);

    var nelat = Map.getBounds().getNorthEast().lat().toFixed(2);
    var nelon = Map.getBounds().getNorthEast().lng().toFixed(2);

    var latcntr = Map.getCenter().lat().toFixed(2);
    var loncntr = Map.getCenter().lng().toFixed(2);

    $.getJSON('/aircraftlist.json?vpn='+nelat+'&vpe='+nelon+
              '&vps='+swlat+'&vpw='+swlon+'&vplat='+latcntr+'&vplon='+loncntr,
              function(data) {
        var stillhere = {}
        for (var j=0; j < data.length; j++) {
            var plane = data[j];
            var marker1 = null;

            if ( (typeof Filters.altitude[0] !== "undefined" && Filters.altitude[0] !== null && plane.alt < Filters.altitude[0]) ||
                 (typeof Filters.altitude[1] !== "undefined" && Filters.altitude[1] !== null && plane.alt > Filters.altitude[1]) ||
                 (typeof Filters.speed[0] !== "undefined" && Filters.speed[0] !== null && plane.spd < Filters.speed[0]) ||
                 (typeof Filters.speed[1] !== "undefined" && Filters.speed[1] !== null && plane.spd > Filters.speed[1]) ||
                 (typeof Filters.distance[0] !== "undefined" && Filters.distance[0] !== null && plane.dis < Filters.distance[0]) ||
                 (typeof Filters.distance[1] !== "undefined" && Filters.distance[1] !== null && plane.dis > Filters.distance[1]) ||
                 (typeof Filters.hex[0] !== "undefined" && Filters.hex[0] !== null && parseInt(plane.hex, 16) < Filters.hex[0]) ||
                 (typeof Filters.hex[1] !== "undefined" && Filters.hex[1] !== null && parseInt(plane.hex, 16) > Filters.hex[1]) )
            {
                continue;
            }

            stillhere[plane.hex] = true;
            plane.fli = $.trim(plane.fli);

            /* update */
            if (Planes[plane.hex]) {
                marker1 = Planes[plane.hex].marker1;

                var newpos = new google.maps.LatLng(plane.lat, plane.lon);

                /* update flight number */
                marker1.setIcon(getIconForPlane(plane));
                marker1.setPosition(newpos);
                marker1.set("labelContent", labelDataTable(plane));
                labelSetAnchor(plane, marker1, 12);
                labelSetStyle(plane, marker1);
                marker1.label.draw();

                if (plane.hex == Selected)
                    refreshSelectedInfo();

            } else {
                /* new flight */
                marker1 = new MarkerWithLabel({
                    position: new google.maps.LatLng(plane.lat, plane.lon),
                        draggable: false,
                        raiseOnDrag: false,
                        zIndex: 70000,
                        map: Map,
                        labelContent: labelDataTable(plane),
                        labelAnchor: new google.maps.Point(0,0),
                        labelClass: "labels",
                        labelStyle: {},
                        labelZIndex: 60000,
                        icon: getIconForPlane(plane)
                });
                labelSetAnchor(plane,marker1,12);
                    labelSetStyle(plane,marker1);

                    /* Trap clicks for this marker. */
                    marker1.addListener('click', selectPlane);
            }

            marker1.planehex = plane.hex;   /* for click */
            plane.marker1 = marker1;
            Planes[plane.hex] = plane;

            if (plane.fli.length == 0) {
                marker1.setTitle(plane.hex)
            } else {
                var title = plane.fli+' ('+plane.hex+')';
                if (plane.org.length && plane.dst.length)
                    title += '\n'+plane.org+'>'+plane.dst;
                    marker1.setTitle(title)
            }
        }
        NumPlanes = data.length;

        /* Remove idle planes. */
        for (var p in Planes) {
            if (! stillhere[Planes[p].hex]) {
                if(Planes[p].hex == Selected) {
                    deselectPlane();
                }
                Planes[p].marker1.setMap(null);
                delete Planes[p];
            }
        }
    });
}

function FilterControl(controlDiv, map) {

    var controlUI = document.createElement('div');
    controlUI.className = 'filter-button';
    controlUI.title = 'Click to modify the filters';
    controlDiv.appendChild(controlUI);
  
    var controlText = document.createElement('div');
    controlText.className = 'filter-icon';
    controlUI.appendChild(controlText);
  
    controlUI.addEventListener('click', function() {
        if (document.getElementById('filter-dialog') === null) {
            var filterDialog = document.createElement('div');
            filterDialog.id = 'filter-dialog';
            filterDialog.innerHTML = '<div class="clear" title="Clear"></div><div class="close" title="Close"></div>' +
            '<table>' + 
            '<tr><td>Altitude</td><td><input id="filter-alt-min" placeholder="from" /></td><td><input id="filter-alt-max" placeholder="to" /></td></tr>' + 
            '<tr><td>Speed</td><td><input id="filter-spd-min" placeholder="from" /></td><td><input id="filter-spd-max" placeholder="to" /></td></tr>' + 
            '<tr><td>Distance</td><td><input id="filter-dis-min" placeholder="from" /></td><td><input id="filter-dis-max" placeholder="to" /></td></tr>' + 
            '<tr><td>Hex</td><td><input id="filter-hex-min" placeholder="from" /></td><td><input id="filter-hex-max" placeholder="to" /></td></tr>' + 
            '<tr><td></td><td></td><td><button id="filter-apply">Apply &amp; Close</button>' + 
            '</table>';
            controlDiv.appendChild(filterDialog);

            function handleChange() {
                Filters.altitude[0] = $("#filter-alt-min").val() || undefined;
                Filters.altitude[1] = $("#filter-alt-max").val() || undefined;
                Filters.speed[0] = $("#filter-spd-min").val() || undefined;
                Filters.speed[1] = $("#filter-spd-max").val() || undefined;
                Filters.distance[0] = $("#filter-dis-min").val() || undefined;
                Filters.distance[1] = $("#filter-dis-max").val() || undefined;
                Filters.hex[0] = parseInt($("#filter-hex-min").val(), 16) || undefined;
                Filters.hex[1] = parseInt($("#filter-hex-max").val(), 16) || undefined;
                setCookie("filters",JSON.stringify(Filters),9999)
            }

            $("#filter-alt-min, #filter-alt-max, " +
              "#filter-spd-min, #filter-spd-max, " +
              "#filter-dis-min, #filter-dis-max, " +
              "#filter-hex-min, #filter-hex-max").change(handleChange);

            $("#filter-dialog .close").click(function() {
                // handleChange();
                $("#filter-dialog").hide();
            });
            $("#filter-dialog .clear").click(function() {
                $("#filter-alt-min, #filter-alt-max, " +
                "#filter-spd-min, #filter-spd-max, " +
                "#filter-dis-min, #filter-dis-max, " +
                "#filter-hex-min, #filter-hex-max").val('');
                handleChange();
            });

            $("#filter-apply").click(function() {
                handleChange();
                $("#filter-dialog").hide();
            });
        }
        else
        {
            $("#filter-dialog").show();
        }

        $("#filter-alt-min").val(Filters.altitude[0]);
        $("#filter-alt-max").val(Filters.altitude[1]);
        $("#filter-spd-min").val(Filters.speed[0]);
        $("#filter-spd-max").val(Filters.speed[1]);
        $("#filter-dis-min").val(Filters.distance[0]);
        $("#filter-dis-max").val(Filters.distance[1]);

	});
}


function initialize_clock()
{
    var date = new Date();
    var zone_offset = date.getTimezoneOffset() / -60;
    var clock_lcl_class = "CoolClock::40:noSeconds:" + zone_offset;
    document.getElementById('clock_lcl').className = clock_lcl_class;
    CoolClock.findAndCreateClocks();
}


function initialize()
{
      /* Initialize clock on 2D map */
      initialize_clock();

      /* INITIALIZE AC_SYMBOL_MAP
       * Aircraft symbol map is sorted in reverse order as otherwise
       * short plane names (e.g., C5) would be selected prior
       * to longer names (e.g., C550) and so the wrong icon
       * would be selected.
       */       
      ac_symbol_map = ac_symbol_map.sort(sortPlaneSymbolList);

      Map = new google.maps.Map(document.getElementById("map"),{
        disableDefaultUI: true,
        center: new google.maps.LatLng(CenterLat, CenterLon),
        zoom: Zoomlevel,
        zoomControl: true,
        zoomControlOptions: {
                position: google.maps.ControlPosition.LEFT_BOTTOM
        },
        scaleControl: false,
        streetViewControl: false 
        }
        );

      Map.mapTypes.set('Grey map', rcmaptype1);
      Map.setMapTypeId('Grey map');

      /* Load position and zoom from Cookie */
      var currZoom = parseInt(getCookie("zoom"));
      if (currZoom > 0) Map.setZoom (currZoom);

      var currLat = getCookie("pos");
      var currLng = getCookie("poslng");
      var currCntr = new google.maps.LatLng (parseFloat(currLat),parseFloat(currLng));
      if (currLat > 0) Map.setCenter (currCntr);

      flightPath = new google.maps.Polyline({
                path: [],
                geodesic: true,
                strokeColor: '#FF00FF',
                strokeOpacity: 1.0,
                strokeWeight: 2
                });

      Map.addListener('click', deselectPlane);
      Map.addListener('rightclick', doRightClick);

      Map.addListener('zoom_changed', function () {
        setCookie("zoom",Map.getZoom(),9999)
        });
      Map.addListener('dragend', function () {
        var currCntr = new google.maps.LatLng;
        currCntr =  Map.getCenter().lat();
        setCookie("pos",currCntr,9999);
        currCntr =  Map.getCenter().lng();
        setCookie("poslng",currCntr,9999)
        });

      var newFilters = getCookie("filters");
      if (newFilters) {
          $.extend(Filters, JSON.parse(newFilters));
      }

      if(ShowRangeRings)
      {
        drawRangeRings();
        RangeCircle[1].addListener('click',deselectPlane);
        RangeCircle[2].addListener('click',deselectPlane);
        RangeCircle[3].addListener('click',deselectPlane);
        RangeCircle[4].addListener('click',deselectPlane);
        RangeCircle[1].addListener('rightclick', doRightClick);
        RangeCircle[2].addListener('rightclick', doRightClick);
        RangeCircle[3].addListener('rightclick', doRightClick);
        RangeCircle[4].addListener('rightclick', doRightClick);
      }

      var filterControlDiv = document.createElement('div');
      var filterControl = new FilterControl(filterControlDiv, Map);
      filterControlDiv.index = 1;
      Map.controls[google.maps.ControlPosition.LEFT_TOP].push(filterControlDiv);

      hideInfoBox();

      /* Setup night/day overlay */
      nite.init(Map);
      window.setInterval(function() {
        nite.setDate(null);
        nite.refresh();
      }, 20000); // every 20s

      /* Setup our timer to poll from the server. */
      window.setInterval(function() {
        fetchData();
        fetchTrackData();
        refreshGeneralInfo();
      }, 2500);
}

